/*
 
     File: MSAU.h
 Abstract: Audio Unit class definition.
  Version: 1.1
 
 Copyright (C) 2014 Apple Inc. All Rights Reserved.
 
 
*/

#include "AUEffectBase.h"
#include "MSAUVersion.h"

#if AU_DEBUG_DISPATCHER
	#include "AUDebugDispatcher.h"
#endif


#ifndef __MSAU_h__
#define __MSAU_h__


#pragma mark ____MSAU Parameters

// parameters

#define HALFSQRT2 0.7071067811865475f


static const float kDefaultValue_ParamOne = 0.0;//.5
static const float kDefaultValue_ParamTwo = 0;

static CFStringRef kParameterOneName = CFSTR("Pan");//name of param
static CFStringRef kParameterTwoName = CFSTR("Encode");//name of param

enum {
	kParam_One =0,
	kNumberOfParameters=1
//    kParam_Two =1,
//	kNumberOfParameters=2
	//Add your parameters here...

};
//_____________________________________________________________________________________
#pragma mark ____MSAU
class MSAU : public AUEffectBase
{
public:
	MSAU(AudioUnit component);
    
    /*! @method Version */
	virtual OSStatus		    Version() { return kMSAUVersion; }
    
    virtual	bool				SupportsTail () { return false; }
    
    bool encoding = false;
/* */   
#if AU_DEBUG_DISPATCHER
	virtual ~MSAU () { delete mDebugDispatcher; }
#endif
	
	virtual	OSStatus			GetParameterValueStrings(AudioUnitScope		inScope,
                                                    AudioUnitParameterID	inParameterID,
                                                    CFArrayRef *			outStrings);
    
	virtual	OSStatus			GetParameterInfo(AudioUnitScope			inScope,
												 AudioUnitParameterID	inParameterID,
												 AudioUnitParameterInfo	&outParameterInfo);
    
	virtual OSStatus			GetPropertyInfo(AudioUnitPropertyID	inID,
												AudioUnitScope		inScope,
												AudioUnitElement	inElement,
												UInt32 &			outDataSize,
												Boolean	&			outWritable );
	
	virtual OSStatus			GetProperty(AudioUnitPropertyID inID,
											AudioUnitScope 		inScope,
											AudioUnitElement 	inElement,
											void *			outData);
	

    
    //Added: constructors, overridden methods, etc...
public:    
    virtual UInt32 SupportedNumChannels (const AUChannelInfo** outInfo);
    virtual OSStatus Initialize();
    virtual void ChangeEncoding(bool x) {encoding = x;}
    virtual bool GetEncoding() {return encoding;};
private:
    //static Float32* prev_v;
protected:  
    
    virtual OSStatus ProcessBufferLists	(AudioUnitRenderActionFlags     &inFlags,
                                 const 	AudioBufferList         &inBufferList,
                                 AudioBufferList                &outBufferList,
                                 UInt32                         inFrames);
};

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


#endif