/*
 
     File: MSAU_CocoaView.m
 Abstract: Audio Unit Cocoa View imlementation.
  Version: 1.0
 
 Disclaimer: IMPORTANT:  This Apple software is supplied to you by Apple
 Inc. ("Apple") in consideration of your agreement to the following
 terms, and your use, installation, modification or redistribution of
 this Apple software constitutes acceptance of these terms.  If you do
 not agree with these terms, please do not use, install, modify or
 redistribute this Apple software.
 
 In consideration of your agreement to abide by the following terms, and
 subject to these terms, Apple grants you a personal, non-exclusive
 license, under Apple's copyrights in this original Apple software (the
 "Apple Software"), to use, reproduce, modify and redistribute the Apple
 Software, with or without modifications, in source and/or binary forms;
 provided that if you redistribute the Apple Software in its entirety and
 without modifications, you must retain this notice and the following
 text and disclaimers in all such redistributions of the Apple Software.
 Neither the name, trademarks, service marks or logos of Apple Inc. may
 be used to endorse or promote products derived from the Apple Software
 without specific prior written permission from Apple.  Except as
 expressly stated in this notice, no other rights or licenses, express or
 implied, are granted by Apple herein, including but not limited to any
 patent rights that may be infringed by your derivative works or by other
 works in which the Apple Software may be incorporated.
 
 The Apple Software is provided by Apple on an "AS IS" basis.  APPLE
 MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE APPLE SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 
 Copyright (C) 2012 Apple Inc. All Rights Reserved.
 
 
*/

#import "MSAU_CocoaView.h"
//this has to change if we add more parameters
enum {
	kParam_One = 0,
    kNumberOfParameters=1
//    kParam_Two = 1,
//	kNumberOfParameters=2
};


//____________________________________________________________________________________________________
#pragma mark ____ LISTENER CALLBACK DISPATCHER ____
void ParameterListenerDispatcher (void *inRefCon, 
                                  void *inObject, 
                                  const AudioUnitParameter *inParameter, 
                                  Float32 inValue) {
	MSAU_CocoaView *SELF = (MSAU_CocoaView *)inRefCon;
    
    [SELF parameterListener:inObject parameter:inParameter value:inValue];
}

NSString *MSAU_GestureSliderMouseDownNotification = @"CAGestureSliderMouseDownNotification";
NSString *MSAU_GestureSliderMouseUpNotification   = @"CAGestureSliderMouseUpNotification";
//____________________________________________________________________________________________________
@implementation MSAU_GestureSlider

/*	We create our own custom subclass of NSSlider so we can do begin/end gesture notification
	We cannot override mouseUp: because it will never be called. Instead we do a clever trick in mouseDown to send mouseUp notifications */
- (void)mouseDown:(NSEvent *)inEvent {
	[[NSNotificationCenter defaultCenter] postNotificationName: MSAU_GestureSliderMouseDownNotification object: self];
	
	[super mouseDown: inEvent];	// this call does not return until mouse tracking is complete
								// once tracking is done, we know the mouse has been released, so we can send our mouseup notification

	[[NSNotificationCenter defaultCenter] postNotificationName: MSAU_GestureSliderMouseUpNotification object: self];
}
	
@end

//____________________________________________________________________________________________________
@implementation MSAU_CocoaView

#pragma mark ____ (INIT /) DEALLOC ____
- (void)dealloc {
    [self removeListeners];
    [super dealloc];
}
//____________________________________________________________________________________________________
#pragma mark ____ PUBLIC FUNCTIONS ____
- (void)setAU:(AudioUnit)inAU {
	// remove previous listeners
	if (mAU) [self removeListeners];
	mAU = inAU;
    
   	mParameter[0].mAudioUnit = inAU;
	mParameter[0].mParameterID = kParam_One;
	mParameter[0].mScope = kAudioUnitScope_Global;
	mParameter[0].mElement = 0;	
    // is this possible?
    /*
    mParameter[1].mAudioUnit = inAU;
    mParameter[1].mParameterID = kParam_Two;
    mParameter[1].mScope = kAudioUnitScope_Global;
    mParameter[1].mElement = 0; //or 1?
     */
	// add new listeners
	[self addListeners];
	
	// initial setup
	[self synchronizeUIWithParameterValues];
}
//____________________________________________________________________________________________________
#pragma mark ____ INTERFACE ACTIONS ____
- (IBAction)iaParam1Changed:(id)sender {
    float p, m, s;
    float floatValue = [sender floatValue];
    
    OSStatus result = AUParameterSet(mParameterListener, sender, &mParameter[0], (Float32)floatValue, 0);
    NSAssert(result == noErr, @"[MSAU_CocoaView iaParam1Changed:] AUParameterSet()");
    
    if (sender == uiParam1Slider) {
        p =   2.*(floatValue+.5);
        m =   ( p > 1. ? (2.-p) : 1.);
        s =   ( p < 1. ? p : 1.);
        
        [uiParam1TextField  setFloatValue:m];//uiParam1TextField
        [uiParam1TextField1 setFloatValue:s];//uiParam1TextField1
    } else {
        [uiParam1Slider setFloatValue:floatValue];
    }
}
//____________________________________________________________________________________________________
//**
#ifdef ENC
- (IBAction)iaEncPressed:(id)sender {
    
    //if(sender.state == 0) [sender setTitle: @"Decode"];
   // if(sender.state == 1) [sender setTitle: @"Encode"];

    int intValue = [sender intValue];
 /*   OSStatus result = AUParameterSet(mParameterListener, sender, &mParameter[1], intValue, 0);
    NSAssert(result == noErr, @"[MSAU_CocoaView iaParam2Changed:] AUParameterSet()");
*/

    if (sender == uiEncButton) {
        //[uiParam2TextField setFloatValue:floatValue];
        if ( intValue == 0 ) {
            //ChangeEncoding(false);
            [sender setState: 0];
            [sender setTitle: @"Decode"];
            [uiParam1Slider setEnabled: true];
        }else{
            //ChangeEncoding(true);
            [sender setTitle: @"Encode"];
            [sender setState: 1];
            [uiParam1TextField  setFloatValue:1.f];
            [uiParam1TextField1 setFloatValue:1.f];
            [uiParam1Slider setFloatValue: 0.f];
            
            [uiParam1Slider setEnabled: false];
            
        }
        [self synchronizeUIWithParameterValues];
    } else {
        ;//[uiParam2Slider setFloatValue:floatValue];
    }
/**/
}
#endif



//____________________________________________________________________________________________________
//_ PLACEHOLDER EXAMPLE _
/*
another -(IBAction)iaParameter2Changed:(id)sender{
 OSStatus result = AUParameterSet(mParameterListener, sender, &mParameter[1?], (Float32)floatValue, 0);
 NSAssert(result == noErr, @"[MSAU_CocoaView iaParam2Changed:] AUParameterSet()"); 
 }
 if (sender == uiParam2Slider) {
 [uiParam2TextField setFloatValue:floatValue];
 } else {
 [uiParam2Slider setFloatValue:floatValue];
 }
 } 
 */
//____________________________________________________________________________________________________
#pragma mark ____ NOTIFICATIONS ____

// This routine is called when the user has clicked on the slider. We need to send a begin parameter change gesture to alert hosts that the parameter may be changing value
-(void) handleMouseDown: (NSNotification *) aNotification {
	if ([aNotification object] == uiParam1Slider) 
    {
		AudioUnitEvent event;
		event.mArgument.mParameter = mParameter[0];
		event.mEventType = kAudioUnitEvent_BeginParameterChangeGesture;
		
		AUEventListenerNotify (NULL, self, &event);		// NOTE, if you have an AUEventListenerRef because you are listening to event notification, 
														// pass that as the first argument to AUEventListenerNotify instead of NULL 
	}
    //perhaps other parameters can also be added
}
//____________________________________________________________________________________________________
-(void) handleMouseUp: (NSNotification *) aNotification {
	if ([aNotification object] == uiParam1Slider) 
    {
		AudioUnitEvent event;
		event.mArgument.mParameter = mParameter[0];
		event.mEventType = kAudioUnitEvent_EndParameterChangeGesture;
	
		AUEventListenerNotify (NULL, self, &event);		// NOTE, if you have an AUEventListenerRef because you are listening to event notification, 
														// pass that as the first argument to AUEventListenerNotify instead of NULL 
	}
    //perhaps other parameters can also be added
}

//____________________________________________________________________________________________________
#pragma mark ____ PRIVATE FUNCTIONS ____
- (void)addListeners {
    OSStatus result = AUListenerCreate(	ParameterListenerDispatcher, self, 
                                    CFRunLoopGetCurrent(), kCFRunLoopDefaultMode, 0.100, // 100 ms
                                    &mParameterListener	);
	NSAssert(result == noErr, @"[MSAU_CocoaView _addListeners] AUListenerCreate()");
	
    int i;
    for (i = 0; i < kNumberOfParameters; ++i) {
        mParameter[i].mAudioUnit = mAU;
        result = AUListenerAddParameter (mParameterListener, NULL, &mParameter[i]);
        NSAssert(result == noErr, @"[MSAU_CocoaView _addListeners] AUListenerAddParameter()");
    }
    
   	[[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(handleMouseDown:) name:MSAU_GestureSliderMouseDownNotification object: uiParam1Slider];
	[[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(handleMouseUp:) name:MSAU_GestureSliderMouseUpNotification object: uiParam1Slider];
    //perhaps other parameters can also be added
}
//____________________________________________________________________________________________________
- (void)removeListeners {
    OSStatus result;
    int i;
    for (i = 0; i < kNumberOfParameters; ++i) {
        result = AUListenerRemoveParameter(mParameterListener, NULL, &mParameter[i]);
        NSAssert(result == noErr, @"[MSAU_CocoaView _removeListeners] AUListenerRemoveParameter()");
    }
    
    result = AUListenerDispose(mParameterListener);
	NSAssert(result == noErr, @"[MSAU_CocoaView _removeListeners] AUListenerDispose()");
    
    [[NSNotificationCenter defaultCenter] removeObserver: self name:MSAU_GestureSliderMouseDownNotification object: uiParam1Slider];
	[[NSNotificationCenter defaultCenter] removeObserver: self name:MSAU_GestureSliderMouseUpNotification object: uiParam1Slider];
}
//____________________________________________________________________________________________________
- (void)synchronizeUIWithParameterValues {
    OSStatus result;
	Float32 value;
    int i;
    
    for (i = 0; i < kNumberOfParameters; ++i) {
        // only has global parameters
        result = AudioUnitGetParameter(mAU, mParameter[i].mParameterID, kAudioUnitScope_Global, 0, &value);
        NSAssert(result == noErr, @"[MSAU_CocoaView synchronizeUIWithParameterValues] (x.1)");
        
        result = AUParameterSet (mParameterListener, self, &mParameter[i], value, 0);
        NSAssert(result == noErr, @"[MSAU_CocoaView synchronizeUIWithParameterValues] (x.2)");
        
        result = AUParameterListenerNotify (mParameterListener, self, &mParameter[i]);
        NSAssert(result == noErr, @"[MSAU_CocoaView synchronizeUIWithParameterValues] (x.3)");
    }
}
//____________________________________________________________________________________________________
#pragma mark ____ LISTENER CALLBACK DISPATCHEE ____
- (void)parameterListener:(void *)inObject parameter:(const AudioUnitParameter *)inParameter value:(Float32)inValue {
    //inObject ignored in this case.
    float p, m, s;
	switch (inParameter->mParameterID) {
		case kParam_One:
            
             p =   2.*(inValue+.5);
             m =   ( p > 1. ? (2.-p) : 1.);
             s =   ( p < 1. ? p : 1.);
             
            [uiParam1Slider setFloatValue:inValue];
            [uiParam1TextField  setFloatValue:m];//uiParam1TextField
            [uiParam1TextField1 setFloatValue:s];//uiParam1TextField1
            
            //[uiParam1TextField setStringValue:[ [NSNumber numberWithFloat:inValue] stringValue]];
            //[uiParam1TextField1 setStringValue:[[NSNumber numberWithFloat:inValue] stringValue]];
            break;
        //perhaps other parameters can also be added
	}
}
@end

